﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Meibo1 {
  public partial class Form1 : Form {
    public Form1() {
      InitializeComponent();
    }

    const string ErrorTitle = "エラー";
    const string DataFileName = "meibo1.csv";
    System.Text.Encoding MyEnconding = System.Text.Encoding.
      GetEncoding("Shift_JIS");
    List<Person> MainData = new List<Person>();
    private bool FirstTime = true;
    private int IndexValue = 0;
    public int Index {
      get { return IndexValue; }
      set {
        if (SaveData()) {
          IndexValue = value;
          ShowData();

          buttonPrevious.Enabled = value != 0;
          buttonNext.Enabled =
            value < MainData.Count - 1;

          //if文で書いた例
          //if (value == 0)
          //  buttonPrevious.Enabled = false;
          //else
          //  buttonPrevious.Enabled = true;
        }
      }
    }

    private bool SaveData() {
      if (FirstTime) {
        FirstTime = false;
        return true;
      }
      try {
        MainData[Index].Birthday =
          DateTime.Parse(textBoxBirthday.Text);
      } catch (Exception ex) {
        MessageBox.Show(ex.Message);
        textBoxBirthday.Text = MainData[Index].
          Birthday.ToShortDateString();
        return false;
      }
      MainData[Index].Name = textBoxName.Text;
      return true;
    }

    private void Form1_FormClosing(object sender,
      FormClosingEventArgs e) {

      if (!SaveData())
        e.Cancel = true;
      else {
        var result = MessageBox.Show("データを" +
          DataFileName + "に保存しますか？",
          "保存の確認", MessageBoxButtons.YesNo);
        if (result ==
          System.Windows.Forms.DialogResult.Yes) {
          var sb = new StringBuilder();
          foreach (Person p in MainData)
            sb.AppendLine(p.Name + ',' +
              p.Birthday.ToShortDateString());
          try {
            System.IO.File.WriteAllText(DataFileName,
              sb.ToString(), MyEnconding);
          } catch (Exception ex) {
            MessageBox.Show(ex.Message, ErrorTitle);
          }
        }
      }
    }

    private void Form1_Load(object sender,
      EventArgs e) {

      LoadData();
      Index = 0;
    }

    private void ShowData() {
      labelPosition.Text =
        "#" + (Index + 1).ToString();
      textBoxName.Text = MainData[Index].Name;
      textBoxBirthday.Text = MainData[Index].
        Birthday.ToShortDateString();

    }

    private void buttonPrevious_Click(
      object sender, EventArgs e) {

      Index--;
    }

    private void buttonNext_Click(
      object sender, EventArgs e) {

      Index++;
    }


    private void LoadData() {
      string input = "";
      try {
        input = System.IO.File.ReadAllText(
          DataFileName, MyEnconding);
      } catch (Exception ex) {
        MessageBox.Show(ex.Message, ErrorTitle);
        Application.Exit();
      }

      var lines = input.Split(new string[] { 
        Environment.NewLine },
        System.StringSplitOptions.RemoveEmptyEntries);
      foreach (var line in lines) {
        var columns = line.Split(',');
        Person p = null;
        try {
          p = new Person(columns[0], columns[1]);
        } catch (Exception ex) {
          MessageBox.Show(DataFileName + "の" +
            (MainData.Count + 1) + "行め「" + line +
            "」に問題があります" +
            Environment.NewLine + ex.Message,
            ErrorTitle);
          Application.Exit();
        }
        if (p != null) MainData.Add(p);
      }
    }
  }
}
