﻿using System;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;

namespace Prime2 {
  public partial class Form1 : Form {
    int[] KnownPrime = new int[] { 
      2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 
      37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 
      79, 83, 89, 97 };
    int SearchedMax = 100;

    public Form1() { InitializeComponent(); }

    private void Form1_Load(object sender, 
      EventArgs e) {

      comboBoxMax.DropDownStyle =
        ComboBoxStyle.DropDownList;
      int[] maxValues = new int[] { 
        1000, 10000, 100000, 1000000, 10000000, 
        100000000, 1000000000, 2000000000 };
      foreach (int maxValue in maxValues)
        comboBoxMax.Items.Add(maxValue.ToString());
      comboBoxMax.SelectedIndex = 0;
    }

    private void buttonSearch_Click(
      object sender, EventArgs e) {

      var sw = new Stopwatch();
      sw.Start();
      var max = int.Parse(
        comboBoxMax.SelectedItem.ToString());
      GetPrime(max);
      OutputKnownPrime();
      sw.Stop();

      var ts = new TimeSpan(
        sw.ElapsedMilliseconds * 10000);
      MessageBox.Show(KnownPrime.Length +
        "個の素数が見付かりました。処理時間は" +
        sw.ElapsedMilliseconds + "ミリ秒（" +
        ts.ToString(@"hh\:mm\:ss") + "）でした");
    }

    private void GetPrime(int searchMax) {
      if (searchMax > 10000 && 
        SearchedMax <= 100) GetPrime(1000);
      if (searchMax > 1000000 && 
        SearchedMax <= 1000)  GetPrime(10000);
      if (searchMax > 100000000 && 
        SearchedMax <= 10000) GetPrime(100000);

      int i, j, limit;
      bool[] isPrime = new bool[searchMax + 1];
      int primeCount = 0;
      var ub = isPrime.GetUpperBound(0);

      for (i = 0; i <= ub; i++) isPrime[i] = true;
      limit = (int)Math.Ceiling(Math.Sqrt(ub));
      for (i = 0; i <= KnownPrime.GetUpperBound(0); 
        i++) {
        if (KnownPrime[i] > limit) break;
        for (j = 2; j <= searchMax / KnownPrime[i]; 
          j++) isPrime[KnownPrime[i] * j] = false;
      }
      for (i = 2; i <= ub; i++) 
        if (isPrime[i]) primeCount++;
      KnownPrime = new int[primeCount];
      j = 0;
      for (i = 2; i <= ub; i++)
        if (isPrime[i]) {
          KnownPrime[j] = i; 
          j++;
        }
      SearchedMax = searchMax;
    }

    private void OutputKnownPrime() {
      var path = @"c:\temp\prime" + 
        SearchedMax.ToString() + ".txt";
      var myEncoding = System.Text.Encoding.
        GetEncoding("Shift_JIS");
      var stWriter = new StreamWriter(
        path, false, myEncoding);
      foreach(var prime in KnownPrime)
        stWriter.WriteLine(prime.ToString());
      stWriter.Close();
    }
  }
}
